---
driver number: 0x00004
---

# GPIO

## Overview

The GPIO driver allows userspace to synchronously control the output and
receive callbacks on changes in the input for a set of GPIO pins.

GPIO pins are indexed in the array starting at 0. The order of the pins and the
mapping between indexes and actual pins is set by the kernel in the board's
main file.

## Command

  * ### Command number: `0`

    **Description**: How many GPIO pins are supported on this board.

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: The number of pins on the board, or `ENODEVICE` if this driver
    is not present on the board.


  * ### Command number: `1`

    **Description**: Enable output on a GPIO pin. After enabling output,
    output-related operations on the pin (`set`, `clear`, `toggle`) are
    available. Using input operations on a pin in this state is undefined.

    **Argument 1**: The index of the GPIO pin to enable output for, starting at
    0.

    **Argument 2**: unused

    **Returns**: SUCCESS if the command was successful, and `EINVAL` if the
    argument refers to a non-existent pin.

  * ### Command number: `2`

    **Description**: Set the output of a GPIO pin (high). Using this command
    without first enabling output is undefined.

    **Argument 1**: The index of the GPIO pin to set, starting at 0.

    **Argument 2**: unused

    **Returns**: `SUCCESS` if the pin index is valid, `EINVAL` otherwise.

  * ### Command number: `3`

    **Description**: Clear the output of a GPIO pin (low). Using this command
    without first enabling output is undefined.

    **Argument 1**: The index of the GPIO pin to clear, starting at 0.

    **Argument 2**: unused

    **Returns**: `SUCCESS` if the pin index is valid, `EINVAL` otherwise.

  * ### Command number: `4`

    **Description**: Toggle the output of a GPIO pin. If the pin was
    previously high, this operation clears it. If it was previously low, sets
    it. Using this command without first enabling output is undefined.

    **Argument 1**: The index of the GPIO pin to toggle, starting at 0.

    **Argument 2**: unused

    **Returns**: `SUCCESS` if the pin index is valid, `EINVAL` otherwise.

  * ### Command number: `5`

    **Description**: Enable and configure input on a GPIO pin. After enabling
    input, input-related operations on the pin (e.g. `read`) are available.
    Using output operations on a pin in this state is undefined.

    **Argument 1**: The index of the GPIO pin to toggle, starting at 0.

    **Argument 2**: requested resistor to attach to the pin: `0` for pull-none,
    `1` for pull-up, or `2` for pull-down.

    **Returns**: `SUCCESS` if the pin index is valid, `EINVAL` if it is
    invalid, and `ENOSUPPORT` if the resistor configuration is not supported by
    the hardware.

  * ### Command number: `6`

    **Description**: Read the current value of a GPIO pin.

    **Argument 1**: The index of the GPIO pin to read, starting at 0.

    **Argument 2**: unused

    **Returns**: `EINVAL` if the index of the pin is invalid, `1` if the value
    of the pin is high or `0` if it is low.

  * ### Command number: `7`

    **Description**: Configure interrupts on a GPIO pin.
    After enabling interrupts, the callback set in subscribe will be called
    when a the pin level changes.
    Using this command without first enabling input is undefined.

    **Argument 1**: The index of the GPIO pin to read, starting at 0.

    **Argument 2**: Indicates which events trigger callbacks: `0` for either
    edge, `1` for rising edge, or `2` for falling edge.

    **Returns**: `SUCCESS` if the pin index is valid, `EINVAL` if it is
    invalid, and `ENOSUPPORT` if an invalid interrupt mode is passed in the
    configuration field of the argument.

## Subscribe

  * ### Subscribe number: `0`

    **Description**: Subscribe a callback that will fire when any GPIO pin whose
    interrupts have been enabled changes level. Registering the callback does
    not have an effect on whether any GPIO pin interrupts are enabled.

    **Callback signature**: The callback receives two arguments. The first is
    the index of the GPIO pin whose level has changed, and the second is value
    of the pin when the interrupt occurred. The second argument has the same
    semantics as the return value for the `read` command: `0` for low, `1` for
    high.

    **Returns**: SUCCESS if the subscribe was successful, ENOMEM if the driver
    cannot support another app, and `EINVAL` if the app is somehow invalid.

## Allow

Unused for the GPIO driver. Will always return `ENOSUPPORT`.

